pub enum FactSourceRegistrationError {
SharedEmptySession {
fact_name: &'static str,
},
AlreadyRegistered {
fact_name: &'static str,
},
InFlight {
fact_name: &'static str,
},
}Expand description
Error returned by non-panicking fact-source registration helpers.
Variants
The session is the process-wide empty session and cannot accept sources.
AlreadyRegistered
A source is already registered for this exact fact key type.
Fields

fact_name: &'static strDiagnostic fact name from FactKey::NAME.
InFlight
Loads for this fact key type are currently in flight.
Fields

fact_name: &'static strDiagnostic fact name from FactKey::NAME.